/** {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}
*:before,
*:after {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}*/
body {
    background: white;
    font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
    font-size: 18px;
    padding-bottom: 8px;
    margin: 0;
    line-height:1.5;
    color: #555;
}
.content {
    margin: 0 auto;
    padding: 0 2em;
    margin-bottom: 50px;
    line-height: 1.6em;
}

.layout {
    padding-top: 1.0em;
    padding-left: 1.0em;
    padding-right: 1.0em;
    border: none;
    background: white;
    text-align: left;
    vertical-align: top;
    width: auto;
    height: auto;
    margin-left: auto;
    margin-right: auto;
}

.layout h1:first-child,
.layout h2:first-child,
.layout h3:first-child {
    margin-top: -0.7em;
}

a {
    line-height:1.1em;
    text-decoration:none;
    color: #2A6EA6;
}

a sup {
    display:inline-block;
}

a:hover {
    border-bottom: 1px gray dotted;
}

.toptitle {
    padding-top:0.2em;
    padding-bottom: 0.2em;
    margin-bottom: 1.5em;
    border-bottom: 1px solid #eee;
    font-size: 3em;
    text-align:center;
    font-weight: 300;
}

.toptitle:empty {
    display: none;
}

.subtitle {
    margin-top: 0.0em;
    margin-bottom: 0.0em;
    padding-top: 0em;
    padding-bottom: 0.1em;
    font-size: 45%;
    text-align:center;
    font-weight: normal;
}

.subtitle:empty {
    display: none;
}

h1,
h2,
h3,
h4,
h5,
h6 {
  font-family: inherit;
  font-weight: 500;
  line-height: 1.1;
  color: inherit;
  border-bottom: 1px solid #eee;
}
h1 small,
h2 small,
h3 small,
h4 small,
h5 small,
h6 small {
    font-weight: normal;
    line-height: 1;
    color: #777;
}
h1,
h2,
h3 {
    margin-top: 20px;
    margin-bottom: 10px;
}
h1 small,
h2 small,
h3 small {
    font-size: 65%;
}
h4,
h5,
h6 {
    margin-top: 10px;
    margin-bottom: 10px;
}
h4 small,
h5 small,
h6 small {
    font-size: 75%;
}
h1 {
  font-size: 36px;
}
h2 {
  font-size: 30px;
}
h3 {
  font-size: 24px;
}
h4 {
  font-size: 18px;
}
h5 {
  font-size: 14px;
}
h6 {
  font-size: 12px;
}
p {
    margin-top: 0.0em;
    margin-bottom: 0.8em;
    padding: 0;
    line-height: 1.3;
    text-align: justify;
}

pre {
  overflow: auto;
  font-family: monospace, monospace;
  font-size: 1em;
  display: block;
  padding: 9.5px;
  margin: 0 0 10px;
  font-size: 13px;
  line-height: 1.42857143;
  color: #333;
  word-break: break-all;
  word-wrap: break-word;
  background-color: #f5f5f5;
  border: 1px solid #ccc;
  border-radius: 4px;
}
code,
kbd,
pre,
samp {
  font-family: Menlo, Monaco, Consolas, "Courier New", monospace;
}

code {
  padding: 2px 4px;
  font-size: 90%;
  color: #c7254e;
  background-color: #f9f2f4;
  border-radius: 4px;
}

.reference {
    font-size: small;
    border-top: 1px solid #c0c0c0;
    position: relative;
    padding: 45px 15px 15px;
    margin: 5px 5px 5px;
}

.reference:after {
    position: absolute;
    top: 15px;
    left: 15px;
    font-size: 12px;
    font-weight: bold;
    text-transform: uppercase;
    letter-spacing: 1px;
    content: "Reference";
}

.footer {
    font-size: small;
    border-top: 1px solid #c0c0c0;
    padding-top: 0.1em;
    margin-top: 4.0em;
}

.footer a {
    color: #80a0b0;
}

.footer-text {
    padding-bottom: 8px;
    text-align: center;
    color: #a0a0a0;
}

ul {
    list-style-position: outside;
}
ul ul {
}
ul ul ul {
}
p + ul, p + ol {
    margin-top: -0.5em;
}

li ul, li ol {
    margin-top: -0.3em;
}

ol {
    list-style-position: outside;
    list-style-type: decimal;
}

ol ol {
    list-style-type: lower-alpha;
}

ol ol ol {
    list-style-type: lower-roman;
}

p + div.codeblock {
    margin-top: -0.6em;
}

.figure img, .figure svg, .video video {
    border: none;
    margin: auto;
    display: block;
    max-width: 100%;
    max-height: 100%;
}
.figure, .video {
    margin: 20px 20px;
}
.caption {
    text-align: center;
    font-family: Arial;
    line-height: 1.45;
    font-size: 95%;
}

.image-left {
    float:left;
}

.image-thumbnail img {
    border: none;
    max-width: 96px;
}

.syntax-inline .hll { background-color: #ffffcc }
.syntax-inline  { display: inline }
.syntax-inline .c { color: #408080; font-style: italic } /* Comment */
.syntax-inline .err { border: 1px solid #FF0000 } /* Error */
.syntax-inline .k { color: #008000; font-weight: bold } /* Keyword */
.syntax-inline .o { color: #666666 } /* Operator */
.syntax-inline .cm { color: #408080; font-style: italic } /* Comment.Multiline */
.syntax-inline .cp { color: #BC7A00 } /* Comment.Preproc */
.syntax-inline .c1 { color: #408080; font-style: italic } /* Comment.Single */
.syntax-inline .cs { color: #408080; font-style: italic } /* Comment.Special */
.syntax-inline .gd { color: #A00000 } /* Generic.Deleted */
.syntax-inline .ge { font-style: italic } /* Generic.Emph */
.syntax-inline .gr { color: #FF0000 } /* Generic.Error */
.syntax-inline .gh { color: #000080; font-weight: bold } /* Generic.Heading */
.syntax-inline .gi { color: #00A000 } /* Generic.Inserted */
.syntax-inline .go { color: #808080 } /* Generic.Output */
.syntax-inline .gp { color: #000080; font-weight: bold } /* Generic.Prompt */
.syntax-inline .gs { font-weight: bold } /* Generic.Strong */
.syntax-inline .gu { color: #800080; font-weight: bold } /* Generic.Subheading */
.syntax-inline .gt { color: #0040D0 } /* Generic.Traceback */
.syntax-inline .kc { color: #008000; font-weight: bold } /* Keyword.Constant */
.syntax-inline .kd { color: #008000; font-weight: bold } /* Keyword.Declaration */
.syntax-inline .kn { color: #008000; font-weight: bold } /* Keyword.Namespace */
.syntax-inline .kp { color: #008000 } /* Keyword.Pseudo */
.syntax-inline .kr { color: #008000; font-weight: bold } /* Keyword.Reserved */
.syntax-inline .kt { color: #B00040 } /* Keyword.Type */
.syntax-inline .m { color: #666666 } /* Literal.Number */
.syntax-inline .s { color: #BA2121 } /* Literal.String */
.syntax-inline .na { color: #7D9029 } /* Name.Attribute */
.syntax-inline .nb { color: #008000 } /* Name.Builtin */
.syntax-inline .nc { color: #0000FF; font-weight: bold } /* Name.Class */
.syntax-inline .no { color: #880000 } /* Name.Constant */
.syntax-inline .nd { color: #AA22FF } /* Name.Decorator */
.syntax-inline .ni { color: #999999; font-weight: bold } /* Name.Entity */
.syntax-inline .ne { color: #D2413A; font-weight: bold } /* Name.Exception */
.syntax-inline .nf { color: #0000FF } /* Name.Function */
.syntax-inline .nl { color: #A0A000 } /* Name.Label */
.syntax-inline .nn { color: #0000FF; font-weight: bold } /* Name.Namespace */
.syntax-inline .nt { color: #008000; font-weight: bold } /* Name.Tag */
.syntax-inline .nv { color: #19177C } /* Name.Variable */
.syntax-inline .ow { color: #AA22FF; font-weight: bold } /* Operator.Word */
.syntax-inline .w { color: #bbbbbb } /* Text.Whitespace */
.syntax-inline .mf { color: #666666 } /* Literal.Number.Float */
.syntax-inline .mh { color: #666666 } /* Literal.Number.Hex */
.syntax-inline .mi { color: #666666 } /* Literal.Number.Integer */
.syntax-inline .mo { color: #666666 } /* Literal.Number.Oct */
.syntax-inline .sb { color: #BA2121 } /* Literal.String.Backtick */
.syntax-inline .sc { color: #BA2121 } /* Literal.String.Char */
.syntax-inline .sd { color: #BA2121; font-style: italic } /* Literal.String.Doc */
.syntax-inline .s2 { color: #BA2121 } /* Literal.String.Double */
.syntax-inline .se { color: #BB6622; font-weight: bold } /* Literal.String.Escape */
.syntax-inline .sh { color: #BA2121 } /* Literal.String.Heredoc */
.syntax-inline .si { color: #BB6688; font-weight: bold } /* Literal.String.Interpol */
.syntax-inline .sx { color: #008000 } /* Literal.String.Other */
.syntax-inline .sr { color: #BB6688 } /* Literal.String.Regex */
.syntax-inline .s1 { color: #BA2121 } /* Literal.String.Single */
.syntax-inline .ss { color: #19177C } /* Literal.String.Symbol */
.syntax-inline .bp { color: #008000 } /* Name.Builtin.Pseudo */
.syntax-inline .vc { color: #19177C } /* Name.Variable.Class */
.syntax-inline .vg { color: #19177C } /* Name.Variable.Global */
.syntax-inline .vi { color: #19177C } /* Name.Variable.Instance */
.syntax-inline .il { color: #666666 } /* Literal.Number.Integer.Long */
.syntax-inline pre {
    background: transparent;
    display: inline;
    white-space: pre-wrap;       /* Since CSS 2.1 */
    white-space: -moz-pre-wrap;  /* Mozilla, since 1999 */
    white-space: -pre-wrap;      /* Opera 4-6 */
    white-space: -o-pre-wrap;    /* Opera 7 */
    word-wrap: break-word;       /* Internet Explorer 5.5+ */
    border: None;
    border-radius: 0;
}
.syntax-inline .lineno {
    color: #aaa;
    padding-right: 15px;
}
.syntax .hll { background-color: #ffffcc }
.syntax  { background: #f8f8f8; }
.syntax .c { color: #408080; font-style: italic } /* Comment */
.syntax .err { border: 1px solid #FF0000 } /* Error */
.syntax .k { color: #008000; font-weight: bold } /* Keyword */
.syntax .o { color: #666666 } /* Operator */
.syntax .cm { color: #408080; font-style: italic } /* Comment.Multiline */
.syntax .cp { color: #BC7A00 } /* Comment.Preproc */
.syntax .c1 { color: #408080; font-style: italic } /* Comment.Single */
.syntax .cs { color: #408080; font-style: italic } /* Comment.Special */
.syntax .gd { color: #A00000 } /* Generic.Deleted */
.syntax .ge { font-style: italic } /* Generic.Emph */
.syntax .gr { color: #FF0000 } /* Generic.Error */
.syntax .gh { color: #000080; font-weight: bold } /* Generic.Heading */
.syntax .gi { color: #00A000 } /* Generic.Inserted */
.syntax .go { color: #808080 } /* Generic.Output */
.syntax .gp { color: #000080; font-weight: bold } /* Generic.Prompt */
.syntax .gs { font-weight: bold } /* Generic.Strong */
.syntax .gu { color: #800080; font-weight: bold } /* Generic.Subheading */
.syntax .gt { color: #0040D0 } /* Generic.Traceback */
.syntax .kc { color: #008000; font-weight: bold } /* Keyword.Constant */
.syntax .kd { color: #008000; font-weight: bold } /* Keyword.Declaration */
.syntax .kn { color: #008000; font-weight: bold } /* Keyword.Namespace */
.syntax .kp { color: #008000 } /* Keyword.Pseudo */
.syntax .kr { color: #008000; font-weight: bold } /* Keyword.Reserved */
.syntax .kt { color: #B00040 } /* Keyword.Type */
.syntax .m { color: #666666 } /* Literal.Number */
.syntax .s { color: #BA2121 } /* Literal.String */
.syntax .na { color: #7D9029 } /* Name.Attribute */
.syntax .nb { color: #008000 } /* Name.Builtin */
.syntax .nc { color: #0000FF; font-weight: bold } /* Name.Class */
.syntax .no { color: #880000 } /* Name.Constant */
.syntax .nd { color: #AA22FF } /* Name.Decorator */
.syntax .ni { color: #999999; font-weight: bold } /* Name.Entity */
.syntax .ne { color: #D2413A; font-weight: bold } /* Name.Exception */
.syntax .nf { color: #0000FF } /* Name.Function */
.syntax .nl { color: #A0A000 } /* Name.Label */
.syntax .nn { color: #0000FF; font-weight: bold } /* Name.Namespace */
.syntax .nt { color: #008000; font-weight: bold } /* Name.Tag */
.syntax .nv { color: #19177C } /* Name.Variable */
.syntax .ow { color: #AA22FF; font-weight: bold } /* Operator.Word */
.syntax .w { color: #bbbbbb } /* Text.Whitespace */
.syntax .mf { color: #666666 } /* Literal.Number.Float */
.syntax .mh { color: #666666 } /* Literal.Number.Hex */
.syntax .mi { color: #666666 } /* Literal.Number.Integer */
.syntax .mo { color: #666666 } /* Literal.Number.Oct */
.syntax .sb { color: #BA2121 } /* Literal.String.Backtick */
.syntax .sc { color: #BA2121 } /* Literal.String.Char */
.syntax .sd { color: #BA2121; font-style: italic } /* Literal.String.Doc */
.syntax .s2 { color: #BA2121 } /* Literal.String.Double */
.syntax .se { color: #BB6622; font-weight: bold } /* Literal.String.Escape */
.syntax .sh { color: #BA2121 } /* Literal.String.Heredoc */
.syntax .si { color: #BB6688; font-weight: bold } /* Literal.String.Interpol */
.syntax .sx { color: #008000 } /* Literal.String.Other */
.syntax .sr { color: #BB6688 } /* Literal.String.Regex */
.syntax .s1 { color: #BA2121 } /* Literal.String.Single */
.syntax .ss { color: #19177C } /* Literal.String.Symbol */
.syntax .bp { color: #008000 } /* Name.Builtin.Pseudo */
.syntax .vc { color: #19177C } /* Name.Variable.Class */
.syntax .vg { color: #19177C } /* Name.Variable.Global */
.syntax .vi { color: #19177C } /* Name.Variable.Instance */
.syntax .il { color: #666666 } /* Literal.Number.Integer.Long */
.syntax pre {
    white-space: pre-wrap;       /* Since CSS 2.1 */
    white-space: -moz-pre-wrap;  /* Mozilla, since 1999 */
    white-space: -pre-wrap;      /* Opera 4-6 */
    white-space: -o-pre-wrap;    /* Opera 7 */
    word-wrap: break-word;       /* Internet Explorer 5.5+ */
    border-left: None;
    border-right: None;
    border-top: 1px solid #ccc;
    border-bottom: 1px solid #ccc;
    border-radius: 0;
}
.syntax .lineno {
    color: #aaa;
    padding-right: 15px;
}

.syntaxtable pre {
    border: None;
    border-radius: 0;
}
.syntaxtable .linenos {
    background: #eee;
    padding-right: 0;
    width: 40px;
}
.syntaxtable .linenos pre {
    background: #eee
}
.syntaxtable .linenodiv pre {
    border: none;
    border-radius: 0;
    border-right: solid 1px;
    color: #aaa;
    word-wrap: normal;
}
.syntaxtable .linenodiv pre code {
    white-space: pre;
}
.syntaxtable .code {
    border: none;
    border-radius: 0;
    word-wrap: normal;
}

table {
    background-color: transparent;
    border-spacing: 0;
    border-collapse: collapse;
    border-top: 1px solid #eee;
    border-bottom: 1px solid #eee;
}

table tr:nth-child(odd) td{
    background: #f7f7f9;
}

td,
th {
    padding: 0;
}

caption {
    padding-top: 8px;
    padding-bottom: 8px;
    color: #777;
    text-align: left;
}

th {
    text-align: left;
}

table {
    width: 100%;
    max-width: 100%;
    margin-bottom: 20px;
}

table > thead > tr > th,
table > tbody > tr > th,
table > tfoot > tr > th,
table > thead > tr > td,
table > tbody > tr > td,
table > tfoot > tr > td {
  padding: 8px;
  line-height: 1.42857143;
  vertical-align: top;
  border-top: 1px solid #ddd;
}

table > thead > tr > th {
    vertical-align: bottom;
    border-bottom: 2px solid #ddd;
}

table > caption + thead > tr:first-child > th,
table > colgroup + thead > tr:first-child > th,
table > thead:first-child > tr:first-child > th,
table > caption + thead > tr:first-child > td,
table > colgroup + thead > tr:first-child > td,
table > thead:first-child > tr:first-child > td {
    border-top: 0;
}

table > tbody + tbody {
    border-top: 2px solid #ddd;
}

table table {
  background-color: #fff;
}

table col[class*="col-"] {
  position: static;
  display: table-column;
  float: none;
}

table td[class*="col-"],
table th[class*="col-"] {
  position: static;
  display: table-cell;
  float: none;
}

.bs-example {
    position: relative;
    padding: 45px 15px 15px;
    margin: 0 -15px 15px;
    border-color: #e5e5e5 #eee #eee;
    border-style: solid;
    border-width: 1px 0;
    -webkit-box-shadow: inset 0 3px 6px rgba(0,0,0,.05);
    box-shadow: inset 0 3px 6px rgba(0,0,0,.05);
}

.bs-demo {
    position: relative;
    padding: 9px 14px;
    margin: 0 -15px 15px;
    border-color: #e5e5e5 #eee #eee;
    border-style: solid;
    border-width: 1px 0;
    -webkit-box-shadow: inset 0 3px 6px rgba(0,0,0,.05);
    box-shadow: inset 0 3px 6px rgba(0,0,0,.05);
    background-color: #f7f7f9;
}

.bs-example > h1 {
    padding-top: 15px;
}

/* Echo out a label for the example */
.bs-example:after {
    position: absolute;
    top: 15px;
    left: 15px;
    font-size: 12px;
    font-weight: bold;
    color: #959595;
    text-transform: uppercase;
    letter-spacing: 1px;
    content: "Example";
}
.bs-example-src,
.bs-demo-rst {
    width: auto;
    padding: 9px 14px;
    margin-bottom: 14px;
    background-color: #f7f7f9;
    margin: -15px -15px 15px;
    border-style: solid;
    border-width: 1px 0 1px;
    border-radius: 0;
    border-color: #e1e1e8;
}
.bs-demo-rst {
    background-color: #fff;
}
/* Tweak display of the code snippets when following an example */
.bs-example + .bs-example-src,
.bs-demo-src + .bs-demo-rst {
    margin: -15px -15px 15px;
    border-width: 0 0 1px;
    border-radius: 0;
}

.info {
  padding: 45px 15px 15px;
  margin: 0 -15px 15px;
  border: 1px solid #eee;
  border-left-width: 5px;
  border-radius: 3px;
  border-left-color: #1b809e;
  position: relative;
}

/* Make the examples and snippets not full-width */
@media (min-width: 768px) {
    .bs-example {
        margin-right: 0;
        margin-left: 0;
        background-color: #fff;
        border-color: #ddd;
        border-width: 1px;
        border-radius: 4px 4px 0 0;
        -webkit-box-shadow: none;
        box-shadow: none;
    }
    .bs-demo {
        margin-right: 0;
        margin-left: 0;
        background-color: #f7f7f9;
        border-color: #ddd;
        border-width: 1px;
        border-radius: 4px 4px 0 0;
        -webkit-box-shadow: none;
        box-shadow: none;
    }
    .bs-demo + .bs-demo-rst {
        margin-top: -16px;
        margin-right: 0;
        margin-left: 0;
        border-width: 1px;
        border-bottom-right-radius: 4px;
        border-bottom-left-radius: 4px;
    }

    .bs-example-src {
        width: auto;
        padding: 9px 14px;
        margin-top: 0px;
        margin-bottom: 14px;
        background-color: #f7f7f9;
        border: 1px solid #e1e1e8;
        border-radius: 4px;
    }
    .bs-example + .bs-example-src {
        margin-top: -16px;
        margin-right: 0;
        margin-left: 0;
        border-width: 1px;
        border-bottom-right-radius: 4px;
        border-bottom-left-radius: 4px;
    }

    .info {
        margin-right: 0;
        margin-left: 0;
        -webkit-box-shadow: none;
        box-shadow: none;
    }
}

/* Tweak content of examples for optimum awesome */
.bs-example > p:last-child,
.bs-example > ul:last-child,
.bs-example > ol:last-child,
.bs-example > blockquote:last-child,
.bs-example > table:last-child {
  margin-bottom: 0;
}

.bs-demo pre,
.bs-example-src pre {
    padding: 0;
    margin-top: 0;
    margin-bottom: 0;
    /*word-break: normal;
    white-space: nowrap;*/
    background-color: transparent;
    border: 0;
}

.bs-example-src pre code {
    font-size: inherit;
    color: #333; /* Effectively the base text color */
}

.bs-example-src pre code:first-child {
    display: inline-block;
    padding-right: 45px;
}


.info h4 {
  margin-top: 0;
  margin-bottom: 5px;
}
.info p:last-child {
  margin-bottom: 0;
}
.info code {
  border-radius: 3px;
}

.info + .info {
  margin-top: -5px;
}

.info h4 {
  color: #1b809e;
}
.info:after {
    position: absolute;
    top: 15px;
    left: 15px;
    font-size: 12px;
    font-weight: bold;
    color: #1b809e;
    text-transform: uppercase;
    letter-spacing: 1px;
    content: "Note";
}

.proof {
  padding: 20px;
  margin: 20px 0;
  padding: 45px 15px 15px;
  margin: 0 15px 15px;
  border: 1px solid #eee;
  border-left-width: 5px;
  border-radius: 3px;
  border-left-color: #1b809e;
  position: relative;
}
.proof:before {
    position: absolute;
    top: 15px;
    left: 15px;
    font-size: 12px;
    font-weight: bold;
    color: #1b809e;
    text-transform: uppercase;
    letter-spacing: 1px;
    content: "Proof";
}
.proof:after {
    position: absolute;
    bottom: 5px;
    right: 5px;
    font-size: 18px;
    font-weight: bold;
    color: #1b809e;
    content: '\25a0';
}

.center_align {
    width: 100%;
    text-align: center;
}

.quote {
  width: auto;
  font-family: Georgia, serif;
  position: relative;
  padding: 29px 20px 20px 45px;
  line-height: 30px;
  text-shadow: 0 1px 1px white;
  display: inline-block;
  left: 50%;
  transform: translateX(-50%);
}

.quote_content:before, .quote_content:after {
  display: inline-block;
  vertical-align: top;
  height: 30px;
  line-height: 48px;
  font-size: 50px;
  opacity: .2;
}

.quote_content:before {
  content: '\201C';
  margin-left: -8px;
  margin-right: 4px;
}

.quote_content:after {
  content: '\201D';
  margin-left: 4px;
  margin-right: -8px;
}

.quote_by {
  display: block;
  font-size: 13px;
  font-style: italic;
  padding-right: 10px;
  text-align: right;
}
